#########################################################################
# COPYRIGHT (C) 2003         EDF R&D              WWW.CODE-ASTER.ORG    #
#                                                                       #
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR         #
# MODIFY IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS        #
# PUBLISHED BY THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE    #
# LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.                       #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,       #
# BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF        #
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      #
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              #
#                                                                       #
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     #
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO : EDF R&D CODE_ASTER,       #
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        #
#########################################################################

# $Id: init.tcl,v 1.44 2003/06/10 06:58:13 mcourtoi Exp $

# namespace astk
#################################################################
namespace eval astk {
   variable astk_version
   variable astk_date
   variable lang
   variable check_quit
   variable root
   variable pathref
   variable recup
   variable rcdir
   variable tmpdir
   variable local_server
   variable fic_serveurs
   variable fic_options
   variable fic_hosts
   variable fic_prefs
   variable fic_prof
   variable fic_outils
   variable fic_print
   variable modprof
   variable cmd

   variable icon

   # Structures de donnes
   # (voir DOC/sdd_'nom de la sdd'.txt pour le dtail)
   variable UL
   variable profil
   variable config
   variable agla
   variable ihm
   variable rex
   variable rex_fiche
   variable sel
   variable msg

   namespace export astk
}

#################################################################
proc init_gene { pathref root } {
   global env
# Ensures that the table library extension is loaded
   set table(library) Tktable[info sharedlibextension]
   if {
      [string match {} [info commands table]]
      && [catch {package require Tktable} err]
      && [catch {load [file join [pwd] $table(library)]} err]
      && [catch {load [file join [pwd] .. unix $table(library)]} err]
      && [catch {load [file join [pwd] .. win $table(library)]} err]
   } {
      error $err
   }
   package require Tktable

# Initialisation des variables partages
   # mettre dbg<>0 pour tracer
   set ashare::dbg 0

   # sortie apres les verifs ?
   set astk::check_quit 0

   #  numero de version de astk et date
   set cvs_v0 "\$Name: v1_0_14 $"
   set cvs_v1 "VDEV"
   catch {regexp {Name: v(.*) } $cvs_v0 mat0 cvs_v1}
   regsub -all "_" $cvs_v1 "." cvs_v
   set cvs_d0 "\$Date: 2003/06/10 06:58:13 $"
   regexp {Date: ([0-9]*)/([0-9]*)/([0-9]*) .*} $cvs_d0 mat0 aa mm jj

   set astk::astk_version "$cvs_v"
   set astk::astk_date "$jj/$mm/$aa"

   # chemin de rfrence (install)
   set ashare::root $root
   set astk::pathref $pathref

   # chargement des modules
   load_modules

# $DISPLAY
   set ashare::DISPLAY ""
   catch {set ashare::DISPLAY $env(DISPLAY)}

# $HOME sur l'interface
   #MC "~" pose problme car exec ne "glob" pas
   set astk::config(-1,home) "~"
   catch { set astk::config(-1,home) $env(HOME) }

   # nom rserv du serveur hbergeant l'interface
   set astk::local_server "Local"
   set astk::config(-1,nom) $astk::local_server
   # IP et login
   set astk::config(-1,nom_complet) ""
   catch { set astk::config(-1,nom_complet) $env(HOSTNAME) }
   if { [string length $astk::config(-1,nom_complet)] == 0 } {
      catch { set astk::config(-1,nom_complet) [exec hostname] }
   }
   set astk::config(-1,login) [whoami]

# machine accessible
   set astk::config(-1,etat) "on"

# chemin des ressources
   set astk::rcdir [file join $astk::config(-1,home) .astkrc]

# rpertoire temporaire
   set astk::tmpdir "/tmp/astk_$astk::config(-1,login)"

#MC_TEST mettre plutot dans config et verifier avant que l'on peut crire
   file mkdir "$astk::rcdir"
   file mkdir "$astk::tmpdir"

   #  dit si on rcupre ou pas les infos de config
   set astk::recup 1

# fichiers
   # liste de la configuration des serveurs
   set astk::fic_serveurs [file join $astk::rcdir config_serveurs]

   # fichiers contenant les prfrences de la machine interface
   set astk::fic_prefs [file join $astk::rcdir prefs]

   # fichier des derniers profils ouverts
   set astk::fic_prof [file join $astk::rcdir lastprof]

   # fichier de configuration des outils
   set astk::fic_outils [file join $astk::rcdir outils]

   # fichier de configuration des imprimantes
   set astk::fic_print [file join $astk::rcdir printers]

   # numro de l'onglet agla pour faciliter si on bouge
   set astk::ihm(nongl_agla) 4

	# taille de la fenetre
	set astk::ihm(larg) 0
	set astk::ihm(haut) 0

	# profil initial
	set astk::ihm(profil_ini) ""
	set astk::ihm(serv_ini) $astk::local_server
   set astk::ihm(etat_satellite) 0

   # styles
   set astk::ihm(style,couleur) std
   set astk::ihm(style,font) std

   # slection
   init_sel

   # definition des frames
   set astk::ihm(menu)        .bar
   set astk::ihm(fenetre)     .fen.fenetre
   set astk::ihm(satellite)   .fen.satellite
   set astk::ihm(popup)       .popup
   set astk::ihm(status)      .status
   set astk::ihm(asjob)       .asjob
   set astk::ihm(log)         .log
   set astk::ihm(interrompre) .interrompre

   # pr-initialisation pour la fenetre des logs
   # potentiellement cras ensuite
   set ashare::fen_log 1
   set astk::ihm(font,txt)     "-Adobe-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*"
   set astk::ihm(font,labbout) "-Adobe-Helvetica-Bold-R-Normal--*-120-*-*-*-*-*-*"
   # fichier des logs
   set ashare::fic_log [file join $astk::rcdir log]
   
   # init pour le rex
   set astk::rex(mcle) {Titre Type "Version emission" Redacteur "Departement Redacteur" "Date Redaction" \
             Correspondant "Departement Correspondant" "Date transmission" "Type d'evolution" \
             Concerne Description "Date presentation EDA" "Reponse preliminaire" "Type de reponse definitive" "Reponse definitive" \
             Correcteur "Date correction" "Date presentation correction EDA" "Version correction" "Date correction effective" \
             "Date visa CEDA" "Date visa CPA" "Numero anomalie" "Stade de traitement" \
             "Num job Cray" "Num job Cray (2)" "Num job Cray (3)" "Num job Cray (4)" "Num job Cray (5)" \
             "ED associee" "EL associee" "AL associee" "Presence fichiers associes" \
             "Numero de gestion" "Delai souhaite" "Gravite" "Date de realisation negociee" "Nombre de jours a facturer" \
             ""}

   # init des listes arguments, paramtres
   set astk::ihm(l_arg) { dbgjeveux  non 261 \
                          rep_outils oui 262 \
                          rep_dex    oui 263 \
                          rep_mat    oui 268 \
                        }
   set astk::ihm(l_par) { nbmaxnook 264 {} \
                          cpresok   265 {RESOK RESNOOK} \
                          facmtps   266 {} \
                        }
}

# initialise la slection
#################################################################
proc init_sel { } {
   set astk::sel(filename) [list]
   set astk::sel(servname) ""
   set astk::sel(liste) ""
   set astk::sel(indice) ""
}

#################################################################
proc load_modules { } {
# icones
   set icons { openR openF new delete edit \
               dir file up refresh comp \
               help }
   set ficon { vaio_openRS vaio_openS vaio_newS vaio_deleteS vaio_editS \
               vaio_dirS vaio_fileS vaio_upS vaio_refreshS vaio_compS
               vaio_helpS }
   set nmod [llength $icons]
   for {set i 0} {$i < $nmod} {incr i} {
      set iconi [lindex $icons $i]
      set fich [lindex $ficon $i]
      append fich ".gif"
      set fich [file join $ashare::root share "icons" $fich]
      set astk::icon($iconi) $fich
      if { [file exists $fich] == 0 } {
         ashare::log "<ERREUR 000> Icon not found : $fich"
         ashare::my_exit 4
      }
   }

# Chargement des modules
   set modul { init_env unites_logiques \
               init_agla \
               fichier options aide \
               ihm_main ihm_commun ihm_liste ihm_etude ihm_execution \
               ihm_agla ihm_satellite ihm_status ihm_outils ihm_rex \
               tools lance_calcul \
               i_serv astk2run_aster }
   set nmod [llength $modul]
   for {set i 0} {$i < $nmod} {incr i} {
      set modi [lindex $modul $i]
      append modi ".tcl"
      set fich [file join $astk::pathref $modi]
      if { [file exists $fich] } {
         source $fich
      } else {
         ashare::log "<ERREUR 000> External module not found : $fich"
         ashare::my_exit 4
      }
   }

# Chargement des modules partags
   set modsh { config preferences tkselecteur remote_tools appli outils \
               messages lire_mcles ihm_tools file_funct print }
   set nmod [llength $modsh]
   for {set i 0} {$i < $nmod} {incr i} {
      set modi [lindex $modsh $i]
      append modi ".tcl"
      set fich [file join $ashare::root share $modi]
      if { [file exists $fich] } {
         source $fich
      } else {
         ashare::log "<ERREUR 000> External module not found : $fich"
         ashare::my_exit 4
      }
   }

# Chargement du module asjob
   set mod { asjob_main init_asjob listej }
   set nmod [llength $mod]
   for {set i 0} {$i < $nmod} {incr i} {
      set modi [lindex $mod $i]
      append modi ".tcl"
      set fich [file join $ashare::root lib ASJOB $modi]
      if { [file exists $fich] } {
         source $fich
      } else {
         ashare::log "<ERREUR 000> External module not found : $fich"
         ashare::my_exit 4
      }
   }
}

#################################################################
proc init_couleur { } {
   if { $astk::ihm(style,couleur) == "std" } {
      # arrire-plan gnral
      set astk::ihm(couleur,background) "#E2E2E2"
      # arrire-plan entry
      set astk::ihm(couleur,entry_background) "#FFFFFF"
      # go
      set astk::ihm(couleur,go) "#E3D4FF"
      # bouton exit
      set astk::ihm(couleur,exit) "#BB3311"
      # bouton
      set astk::ihm(couleur,bout_off) "#E3D4FF"
      # bouton actif
      set astk::ihm(couleur,bout_on) "#AB8EFF"
   } elseif { $astk::ihm(style,couleur) == "orig" } {
      # arrire-plan gnral
      set astk::ihm(couleur,background) "#E2E2E2"
      # arrire-plan entry
      set astk::ihm(couleur,entry_background) "#FFFFFF"
      # go
      set astk::ihm(couleur,go) "#66EE22"
      # bouton exit
      set astk::ihm(couleur,exit) "#BB3311"
      # bouton
      set astk::ihm(couleur,bout_off) LightGoldenRod2
      # bouton actif
      set astk::ihm(couleur,bout_on) LightGoldenRod3
   } elseif { $astk::ihm(style,couleur) == "nb" } {
      # arrire-plan gnral
      set astk::ihm(couleur,background) "#EEEEEE"
      # arrire-plan entry
      set astk::ihm(couleur,entry_background) "#FFFFFF"
      # go
      set astk::ihm(couleur,go) "#FFFFFF"
      # bouton exit
      set astk::ihm(couleur,exit) "#BB3311"
      # bouton
      set astk::ihm(couleur,bout_off) "#CCCCCC"
      # bouton actif
      set astk::ihm(couleur,bout_on) "#AAAAAA"
   }
   # option tk
   option add *foreground black
   option add *background $astk::ihm(couleur,background)
   option add *Entry*background $astk::ihm(couleur,entry_background)
   option add *HighlightThickness 0
}

#################################################################
proc init_font { } {
# font pour astk
   set font_small             "-Adobe-Helvetica-Blod-R-Normal--*-100-*-*-*-*-*-*"
   set font_small_reg         "-Adobe-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*"
   set font_med               "-Adobe-Helvetica-Bold-R-Normal--*-120-*-*-*-*-*-*"
   set font_med_reg           "-Adobe-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*"
   set font_big               "-Adobe-Helvetica-Bold-R-Normal--*-150-*-*-*-*-*-*"
   set font_courier_small_reg "-*-courier-medium-R-Normal-*-100-*-*-*-*-*-*"
   set font_courier_small     "-*-courier-bold-R-Normal-*-100-*-*-*-*-*-*"
   set font_courier_med_reg   "-*-courier-medium-R-Normal-*-120-*-*-*-*-*-*"
   set font_courier_med       "-*-courier-bold-R-Normal-*-120-*-*-*-*-*-*"
   set font_fixed_small       "-*-fixed-bold-R-Normal-*-100-*-*-*-*-*-*"
   if { $astk::ihm(style,font) == "std" } {
      # labels
      set astk::ihm(font,lab) $font_med_reg
      # labels "petits"
      set astk::ihm(font,labpt) $font_small_reg
      # valeurs texte
      set astk::ihm(font,val) $font_med_reg
      # titre
      set astk::ihm(font,tit) $font_big
      # texte
      set astk::ihm(font,txt) $font_med_reg
      # texte fixed
      set astk::ihm(font,txtfix) $font_courier_small_reg
      # zone de texte fixed
      set astk::ihm(font,zonfix) $font_courier_med_reg
      # labels des listes
      set astk::ihm(font,lablst) $font_med_reg
      # labels menu
      set astk::ihm(font,labmenu) $font_med_reg
      # labels menu gras
      set astk::ihm(font,labmenuB) $font_med
      # labels boutons
      set astk::ihm(font,labbout) $font_med
      # liste de fichiers dans le browser
      set astk::ihm(font,brwlst) $font_med_reg
   } elseif { $astk::ihm(style,font) == "debug" } {
      # labels
      set astk::ihm(font,lab) $font_small
      # valeurs texte
      set astk::ihm(font,val) $font_small
      # titre
      set astk::ihm(font,tit) $font_med
      # texte
      set astk::ihm(font,txt) $font_small
      # zone de texte fixed
      set astk::ihm(font,zonfix) $font_courier_small_reg
      # labels des listes
      set astk::ihm(font,lablst) $font_small
      # labels menu
      set astk::ihm(font,labmenu) $font_small
      # labels boutons
      set astk::ihm(font,labbout) $font_small
   }
# pour les dialog tk
   option add *font $astk::ihm(font,lab)
}

# dfinition des listes de mots_cles dans les fichiers de config
#################################################################
proc init_mots { } {
# fichier de config
   ashare::init_mots_config
# fichier de messages
   ashare::init_mots_msg
# fichier prefs
   set ashare::mots(MCS_pref) { nom_user email cua org def_vers xterm editeur \
                                nb_reman langue dbglevel freq_actu nb_ligne \
                                flag_maj }
# fichier lastprof
   set ashare::mots(MCF_prof) { profil }
   set ashare::mots(SSF_prof) { serv }
# fichier outils
   set ashare::mots(MCF_out) { nom }
   set ashare::mots(SSF_out) { cmde ext dist }
# fichier des imprimantes
   set ashare::mots(MCF_pr) { label }
   set ashare::mots(SSF_pr) { serv file cmde }
}

# liste des plates-formes : correspondance nom "aster" - nom parlant
#################################################################
proc init_pltf { } {
   set astk::ihm(pltf,P_LINUX) "Linux x86"
   set astk::ihm(pltf,FREEBSD) "FreeBSD alpha"
   set astk::ihm(pltf,TRU64) "OSF1 TRU64"
   set astk::ihm(pltf,SOLARIS) "Solaris"
   set astk::ihm(pltf,SOLARIS64) "Solaris 64 bits"
   set astk::ihm(pltf,IRIX64) "IRIX 64 bits"
   set astk::ihm(pltf,HPUX) "HP-UX"
   set astk::ihm(pltf,PPRO_NT) "Windows"
}

#################################################################
proc set_titre { } {
   set titre [ashare::mess ihm 107 $astk::astk_version]
   append titre " - "
   append titre [file tail $astk::profil(nom_profil)]
   if { $astk::modprof } {
      append titre " "
      append titre [ashare::mess ihm 133]
   }
   if { [file tail $astk::profil(nom_profil)] != [ashare::mess ihm 27] } {
      append titre " - "
      append titre [file dirname $astk::profil(nom_profil)]
   }
   wm title . $titre
}

# init_profil : reinitialise la config/options
# appele aussi par Fichier/Nouveau
#################################################################
proc init_profil {} {
# onglets
   for {set i 1} {$i <= $astk::ihm(nbongM)} {incr i} {
      for {set j 1} {$j <= $astk::ihm(nbong,$i) } {incr j} {
         set astk::profil($astk::ihm(ong,$i,$j)) non
      }
   }

   set astk::profil(make_etude) oui

   set astk::profil(etude,nbfic) 0
   set astk::profil(path_etude) ""
   set astk::profil(serv_etude) -1
   set astk::profil(tests,nbfic) 0
   set astk::profil(path_tests) ""
   set astk::profil(serv_tests) -1
   set astk::profil(sources,nbfic) 0
   set astk::profil(path_sources) ""
   set astk::profil(serv_sources) -1
   set astk::profil(surcharge,nbfic) 0
   set astk::profil(path_surcharge) ""
   set astk::profil(serv_surcharge) -1
   set astk::profil(args) ""
   set astk::profil(special) ""

   set astk::profil(nb_proc) 1
   set astk::profil(debug) 0
   
   set astk::profil(serv_profil) -1
   set astk::profil(nom_profil) [ashare::mess ihm 27]
   set astk::profil(serv_fich_export) -1
   set astk::profil(nom_fich_export) "_VIDE"

# arguments/options
   set narg [expr [llength $astk::ihm(l_arg)] / 3]
   for {set i 0} {$i < $narg} {incr i} {
      set opt  [lindex $astk::ihm(l_arg) [expr $i * 3]]
      set astk::profil(option,$opt) 0
      set astk::profil(opt_val,$opt) ""
   }
   set npar [expr [llength $astk::ihm(l_par)] / 3]
   for {set i 0} {$i < $npar} {incr i} {
      set opt  [lindex $astk::ihm(l_par) [expr $i * 3]]
      set astk::profil(option,$opt) 1
      switch $opt {
         nbmaxnook { set val 5 }
         cpresok   { set val RESNOOK }
         facmtps   { set val 1 }
         default   { set val "" }
      }
      set astk::profil(opt_val,$opt) $val
   }

# flag de modification du profil
   modprof off
}

# raccourci pour signaler qu'on modifie le profil
#################################################################
proc modprof { {mode on} } {
   if { $mode == "on" } {
      set astk::modprof 1
   } else {
      set astk::modprof 0
   }
   set_titre
   return 1
}

