#########################################################################
# COPYRIGHT (C) 2003         EDF R&D              WWW.CODE-ASTER.ORG    #
#                                                                       #
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR         #
# MODIFY IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS        #
# PUBLISHED BY THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE    #
# LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.                       #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,       #
# BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF        #
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      #
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              #
#                                                                       #
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     #
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO : EDF R&D CODE_ASTER,       #
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        #
#########################################################################

# $Id: init_env.tcl,v 1.29 2003/05/27 16:11:51 mcourtoi Exp $

#################################################################
proc init_env { {parent ""} } {
# lecture de la configuration des serveurs
   init_config

   for {set i 0} {$i < $astk::config(nb_serv)} {incr i} {
      if { $astk::recup && $astk::config($i,recup) == "auto"
        && $astk::config($i,etat) == "on" } {
         # rcupration automatique de la config si plus de n jours
         set hh [clock seconds]
         set lrec [clock scan $astk::config($i,last_recup)]
         set nbj [expr ($hh-$lrec)/24/3600]
         if { $nbj > 30 } {
            recup_info $parent $i
         }
      }
   }
   # ncessaire s'il y a eu une erreur + last_recup + nouveaux versions/noeuds
   ashare::save_options

   # fournit la liste  l'ihm
   set astk::profil(noeud) "-"
	cree_liste_machines

# serveur de profil_ini
   if { $astk::ihm(serv_ini) != "" } {
      set astk::ihm(serv_ini) $astk::inv(serv,$astk::ihm(serv_ini))
   }

# initialisation des outils et imprimantes
   init_outils
   init_print

# derniers profils ouverts
   lire_prof
   cohe_prof
}

#################################################################
proc init_config {} {
   set fich $astk::fic_serveurs
   set iret [ ashare::lire_options $fich astk::config ]
   if { $ashare::dbg } {
      ashare::log "<DEBUG> (lire_options) retour : $iret"
   }
   if { $iret != 0 } {
      set fich [file join $astk::pathref astkrc config_serveurs]
      if { $iret == "FICH_NON_TROUVE" } {
         ashare::mess "info" 4 $astk::fic_serveurs
         ashare::lire_options $fich astk::config
      } elseif { $iret == "PB_astkrc_version" } {
         ashare::mess "erreur" 4 $astk::fic_serveurs.old
         file rename -force $astk::fic_serveurs $astk::fic_serveurs.old
         ashare::lire_options $fich astk::config
      } else {
         ashare::mess "erreur" 1 "ashare::lire_options"
         ashare::my_exit 4
      }
   }
}

# Rcupre les infos sur le serveur
#################################################################
proc recup_info { parent serv } {
   set iret -1
   ashare::mess "info" 12 $astk::config($serv,nom)
# prparation de la ligne de commande  excuter
   set lcmd [file join $astk::config($serv,rep_serv) as_serv]
#  execution
   set iret [ ashare::rexec_cmd $serv astk::config $lcmd "as_info" 0 out $parent]
   set jret $iret
   if { $jret == 0 } {
      if { $ashare::dbg >= 5 } {
         ashare::log "<DEBUG> (recup_info) output as_info = $out"
      }
      # decodage du retour
      if { [regexp {@PARAM@(.*)@FINPARAM@}       $out mat1 lpar] != 1 ||
           [regexp {@VERSIONS@(.*)@FINVERSIONS@} $out mat1 lver] != 1 ||
           [regexp {@NOEUDS@(.*)@FINNOEUDS@}     $out mat1 lnod] != 1 } {
         set jret 1
      } else {
      # paramtres
         set nocc [expr [llength $lpar] / 3]
         for {set k 0} {$k < $nocc } {incr k} {
            regsub {^[ ]*} [lindex $lpar [expr $k * 3]] "" mc
            regsub {[ ]*$} $mc "" mc
            regsub {^[ ]*} [lindex $lpar [expr $k * 3 + 2]] "" val
            regsub {[ ]*$} $val "" val
# glut pour le temps
            if { [regexp {tpsmax} $mc mat1] == 1 } {
               set val [conv_tps $val]
            }
            set astk::config($serv,$mc) $val
            if { $ashare::dbg >= 3 } {
               ashare::log "<DEBUG> (recup_info) astk::config($serv,$mc) = $val"
            }
         }
      # versions
         array unset astk::config($serv,vers,*)
         set astk::config($serv,nb_vers) 0
         set nocc [expr [llength $lver] / 3]
         for {set k 0} {$k < $nocc } {incr k} {
            regsub {^[ ]*} [lindex $lver [expr $k * 3 + 2]] "" val
            regsub {[ ]*$} $val "" val
            set astk::config($serv,vers,$astk::config($serv,nb_vers)) $val
            if { $ashare::dbg >= 3 } {
               ashare::log "<DEBUG> (recup_info) astk::config($serv,vers,$astk::config($serv,nb_vers)) = $val"
            }
            incr astk::config($serv,nb_vers)
         }
      # noeuds
         array unset astk::config($serv,noeud,*)
         set astk::config($serv,nb_noeud) 0
         set nocc [expr [llength $lnod] / 3]
         for {set k 0} {$k < $nocc } {incr k} {
            regsub {^[ ]*} [lindex $lnod [expr $k * 3 + 2]] "" val
            regsub {[ ]*$} $val "" val
            set astk::config($serv,noeud,$astk::config($serv,nb_noeud)) $val
            if { $ashare::dbg >= 3 } {
               ashare::log "<DEBUG> (recup_info) astk::config($serv,noeud,$astk::config($serv,nb_noeud)) = $val"
            }
            incr astk::config($serv,nb_noeud)
         }
      # machine de rfrence ?
         set astk::config($serv,mach_ref) "non"
         if { [regexp {@MACHREF@(.*)@FINMACHREF@} $out mat1 lmach] } {
            set astk::config($serv,mach_ref) "oui"
            if { [regexp {@ROLE@\n(.*)\n@FINROLE@} $out mat1 lrole] } {
               set astk::agla(instance) [split $lrole]
            }
            if { [regexp {@INFOID@\n(.*)\n@FININFOID@} $out mat1 ldet] } {
               set astk::agla(infoid) [split $ldet]
            }
            if { [regexp {@CUA@\n(.*)\n@FINCUA@} $out mat1 lcua] } {
               set astk::agla(list_cua) [split $lcua]
            }
            if { [regexp {@REX_EMET_NOM@\n(.*)\n@FINREX_EMET_NOM@} $out mat1 lnom] } {
               regsub -all " " $lnom "" lnom
               set astk::rex(list_emet_nom) [split $lnom]
            }
            if { [regexp {@REX_EMET_DPT@\n(.*)\n@FINREX_EMET_DPT@} $out mat1 lnom] } {
               regsub -all " " $lnom "" lnom
               set astk::rex(list_emet_dpt) [split $lnom]
            }
            if { [regexp {@REX_CORRESP_NOM@\n(.*)\n@FINREX_CORRESP_NOM@} $out mat1 lnom] } {
               regsub -all " " $lnom "" lnom
               set astk::rex(list_corresp_nom) [split $lnom]
            }
            if { [regexp {@REX_CORRESP_DPT@\n(.*)\n@FINREX_CORRESP_DPT@} $out mat1 lnom] } {
               regsub -all " " $lnom "" lnom
               set astk::rex(list_corresp_dpt) [split $lnom]
            }
            if { [regexp {@REX_CORRECT_NOM@\n(.*)\n@FINREX_CORRECT_NOM@} $out mat1 lnom] } {
               regsub -all " " $lnom "" lnom
               set astk::rex(list_correct_nom) [split $lnom]
            }
            if { [regexp {@REX_CORRECT_DPT@\n(.*)\n@FINREX_CORRECT_DPT@} $out mat1 lnom] } {
               regsub -all " " $lnom "" lnom
               set astk::rex(list_correct_dpt) [split $lnom]
            }
         }
      # version des services
         set client_vers "00.00.00"
         set serv_vers "00.00.00"
         if { [regexp {([0-9]+)\.([0-9]+)\.([0-9]+)} $astk::astk_version mat1 i1 i2 i3] } {
            set client_vers [format "%02d.%02d.%02d" $i1 $i2 $i3]
         }
         set serv_vers "00.00.00"
         regexp {@SERV_VERS@\n(.*)\n@FINSERV_VERS@} $out mat1 serv_vers
         set icmp [string compare $serv_vers $client_vers]
         if { $icmp < 0 } {
           tk_messageBox -title [ashare::mess ihm 138] -message [ashare::mess ihm 233 $astk::config($serv,nom) $serv_vers $client_vers] -type ok -icon info
         } elseif { $icmp > 0 } {
           tk_messageBox -title [ashare::mess ihm 138] -message [ashare::mess ihm 234 $astk::config($serv,nom) $serv_vers $client_vers] -type ok -icon info
         }
      # mise  jour de last_recup (ATTENTION : format anglosaxon)
         set hh [clock format [clock seconds] -format "%m/%d/%Y"]
         set astk::config($serv,last_recup) $hh
      }
   } else {
      ashare::mess "erreur" 3 "as_info" $iret $out
   }
   ashare::mess "info" 13
   return $jret
}

# Lecture de la liste des derniers profils ouverts
#################################################################
proc lire_prof { } {
   set fich "$astk::fic_prof"
   set astk::ihm(prof,nb) 0
   if { $ashare::dbg >= 2 } {
      ashare::log "<DEBUG> (lire_prof) Lecture $fich" }

   if { [ file exists $fich ] == 0 } {
      return 0
   }
   set iret [ ashare::lire_mc_val $fich mots vale nlu ]
   if { $iret != 0 } {
      return $iret }
 
   # phase de vrif
   # mots-cls reconnus
   set mots_cles $ashare::mots(MCF_prof)
   # mots-cls sous le mot-cl facteur
   set mots_smcf $ashare::mots(SSF_prof)
   append mots_cles $mots_smcf
   # les mots-cls sont obligatoires
   set pmcs [llength $mots_smcf]
   set pmcf [expr [llength $mots_cles] - $pmcs - 1]
   for { set k 0 } { $k < [ llength $mots_cles ] } { incr k } {
      set mcs [lindex $mots_cles $k]
      set pres($mcs) 0
   }

   set nbmcf -1
   set nberr 0
   for {set j 1} {$j <= $nlu} {incr j} {
      #astkrc_version
      if { $mots($j) == "astkrc_version" } {
         if { $vale($j) != "$ashare::astkrc_version" } {
            ashare::mess "erreur" 2 $fich
            return PB_astkrc_version
         }
      } else {
         set iv [lsearch -exact $mots_cles $mots($j)]
         if { $iv < 0 } {
            ashare::mess "erreur" 5 $mots($j)
            incr nberr
         } else {
            incr pres($mots($j))
            if { $mots($j) == "profil" } {
               incr nbmcf
               set astk::ihm(prof,$nbmcf) $vale($j)
            } else {
               set astk::ihm(prof,$mots($j),$nbmcf) $vale($j)
            }
         }
      }
   }
   if { $nbmcf < 0 } {
      ashare::mess "erreur" 7 "profil" "profile" ""
   }
   incr nbmcf
   set astk::ihm(prof,nb) $nbmcf
   # est-ce qu'il manque un mot-cl ?
   set sum 0
   for { set k 0 } { $k < [ llength $mots_cles ] } { incr k } {
      set mcs [lindex $mots_cles $k]
      set sum [expr $sum + $pres($mcs)]
   }
   if { $sum != [expr $pmcf+($nbmcf*($pmcs+1))] } {
      ashare::mess "erreur" 6 "?" "?" $fich ""
      incr nberr
   }
   if { $nberr > 0 } {
      ashare::mess "erreur" 8 $fich
      ashare::my_exit 4
   }
   if { $ashare::dbg >= 2 } {
      ashare::log "<DEBUG> (lire_prof) $nbmcf profils" }
   return 0
}

# construit la liste des noeuds accessibles par l'ihm : ihm(lnoeud,$i)
# et les tableaux inverses
#################################################################
proc cree_liste_machines { } {
   set pastrouve 1
	set i 0
   set astk::ihm(lnoeud,$i) "-"
   set astk::inv(serv,-) 0
   set astk::inv(serv,$astk::config(-1,nom)) -1
   set astk::inv(cmpl,$astk::config(-1,nom_complet),$astk::config(-1,login)) -1
   for { set is 0 } { $is < $astk::config(nb_serv) } { incr is } {
      set astk::inv(serv,$astk::config($is,nom)) $is
      set astk::inv(cmpl,$astk::config($is,nom_complet),$astk::config($is,login)) $is
      for { set in 0 } { $in < $astk::config($is,nb_noeud) } { incr in } {
         set deja 0
         for {set k 0} {$k < $i} {incr k} {
            if { $astk::ihm(lnoeud,$k) == $astk::config($is,noeud,$in) } {
               set deja 1
            }
         }
         if { $deja } {
            ashare::mess info 28 $astk::config($is,noeud,$in) $astk::config($is,nom)
         } else {
	         set astk::ihm(lnoeud,$i) $astk::config($is,noeud,$in)
            if { $astk::profil(noeud) == $astk::ihm(lnoeud,$i) } {
               set pastrouve 0
	         }
	         incr i
         }
		}
	}
	set astk::ihm(nb_noeud) $i
# il faut rinitialiser serveur/noeud/version
   if { $pastrouve && $i > 0 } {
	   set astk::profil(noeud) $astk::ihm(lnoeud,0)
   }
# debug niveau 3
	if { $ashare::dbg >= 5 } {
   	for { set is 0 } { $is < $astk::config(nb_serv) } { incr is } {
			ashare::log "<DEBUG> (cree_liste_machines) astk::inv(serv,$astk::config($is,nom)) = $is"
		}
	}
}

# retourne le serveur connect  un noeud particulier
#################################################################
proc quel_serveur { noeud } {
	set i 0
   for { set is 0 } { $is < $astk::config(nb_serv) } { incr is } {
      set serv $astk::config($is,nom)
      for { set in 0 } { $in < $astk::config($is,nb_noeud) } { incr in } {
			if { $astk::config($is,noeud,$in) == $noeud } {
			   if { $ashare::dbg } {
					ashare::log "<DEBUG> (quel_serveur) $noeud est connect  $serv" }
				return $is
			}
		}
	}
	# si on ne l'a pas trouv
	if { $ashare::dbg } {
		ashare::log "<DEBUG> (quel_serveur) $noeud non trouv" }
	return "-"
}

# vrifie qu'un serveur existe
# retourne 1 si le serveur est toujours dans la liste, 0 sinon
# champ vaut nom ou nom_complet selon le test voulu
#################################################################
proc valid_serveur { serv {champ nom}} {
	set iret 0
   if { $serv == $astk::local_server && $champ == "nom" } {
      set iret 1
   } else {
      for { set is 0 } { $is < $astk::config(nb_serv) } { incr is } {
         if { $serv == $astk::config($is,$champ) } {
            set iret 1
            break
         }
	   }
   }
	# si on ne l'a pas trouv
	if { $ashare::dbg && $iret == 0 } {
		ashare::log "<DEBUG> (valid_serveur) $serv n'existe plus"
   }
	return $iret
}

# vrifie les serveurs des profils rmanents
#################################################################
proc cohe_prof { } {
   set nb 0
   for {set i 0} {$i < $astk::ihm(prof,nb)} {incr i} {
      if { [valid_serveur $astk::ihm(prof,serv,$i)] } {
         set astk::ihm(prof,$nb) $astk::ihm(prof,$i)
         set astk::ihm(prof,serv,$nb) $astk::ihm(prof,serv,$i)
         incr nb
      }
   }
   set astk::ihm(prof,nb) $nb
}

# vrifie que :
#   - les serveurs des fichiers d'un profil sont tjs dans la config
#   - le type des fichiers existe toujours
#################################################################
proc cohe_fich { } {
   set err 0
   foreach var { etude tests sources surcharge } {
      for {set i 0} {$i < $astk::profil($var,nbfic)} {incr i} {
         if { [valid_serveur $astk::profil($var,fich,$i,serv)] == 0 } {
            ashare::mess info 18 $astk::profil($var,fich,$i,serv) $astk::local_server
            set astk::profil($var,fich,$i,serv) $astk::local_server
            incr err
         }
      }
   }
   if { $err > 0 } {
      return 2
   }
   return 0
}
