#########################################################################
# COPYRIGHT (C) 2003         EDF R&D              WWW.CODE-ASTER.ORG    #
#                                                                       #
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR         #
# MODIFY IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS        #
# PUBLISHED BY THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE    #
# LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.                       #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,       #
# BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF        #
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      #
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              #
#                                                                       #
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     #
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO : EDF R&D CODE_ASTER,       #
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        #
#########################################################################

# $Id: tools.tcl,v 1.17 2003/05/21 11:29:48 mcourtoi Exp $

# verifications avant lancement
# versions run_aster, display
# si sortie=1, on quitte le programme immediatement apres
#################################################################
proc check_conf { sortie } {
   set nberr 0
# info sur le DISPLAY, warning si :0 ou :0.0 ou ajouter uname -n devant
   if { $ashare::DISPLAY == ":0" || $ashare::DISPLAY == ":0.0" } {
      set ashare::DISPLAY "$astk::config(-1,nom_complet):0.0"
   }
   ashare::mess info 23 $ashare::DISPLAY
   # warning si pas de nom de domaine
   if { [regexp -- {.*\..*\..*:.*} $ashare::DISPLAY] == 0 } {
      ashare::mess info 27
   }

# vrifie : ps -p NUM_PROCESS
# on ne connait pas de NUM_PROCESS en cours, on teste avec -o,
# car le ps qui n'accepte pas -p, n'accepte pas non plus -o...
   set possib { ps /bin/ps /usr/bin/ps }
   for {set i 0} {$i < [llength $possib]} {incr i} {
      set astk::cmd(ps) [lindex $possib $i]
      set cmd "exec /bin/sh -c \"$astk::cmd(ps) -o pid 1> /dev/null 2>&1 ; echo $?\""
      catch { eval $cmd } out
      if { $out != 0 } {
         set astk::cmd(ps) -1
      } else {
         break
      }
   }
   if { $astk::cmd(ps) == -1 } {
      ashare::mess erreur 23 ps
      incr nberr
   }
   ashare::mess info 26 ps $astk::cmd(ps)

   if { $sortie } {
# vrifie que les commandes distantes fonctionnent
      init_config
      set cmd "rsh -n -l $astk::config(-1,login) $astk::config(-1,nom_complet) echo hello"
      for {set i 0} {$i < $astk::config(nb_serv)} {incr i} {
         if { $astk::config($i,etat) == "on" } {
            ashare::mess info 29 $astk::config(-1,nom_complet) $astk::config($i,nom_complet)
            set iret [ashare::rexec_cmd $i astk::config "echo hello" "" 0 out .]
            if { $iret == 0 && $out == "hello" } {
               ashare::mess info 30
               # test le sens inverse
               ashare::mess info 29 $astk::config($i,nom_complet) $astk::config(-1,nom_complet)
               set jret [ashare::rexec_cmd $i astk::config $cmd "" 0 out .]
               if { $jret == 0 && $out == "hello" } {
                  ashare::mess info 30
               } else {
                  ashare::mess erreur 25 $astk::config(-1,login) $astk::config(-1,nom_complet) $astk::config($i,login) $astk::config($i,nom_complet) $out
                  incr nberr
               }
            } else {
               ashare::mess erreur 25 $astk::config($i,login) $astk::config($i,nom_complet) $astk::config(-1,login) $astk::config(-1,nom_complet) $out
               incr nberr
            }
         }
      }
   }

# quitte ?
   ashare::mess "info" 11 $nberr
   if { $sortie || $nberr > 0 } {
      ashare::my_exit 1
   }
}

# convertit astk::profil(temps) de "00:00:00" en secondes
# 00:00 correspond  mm:ss
# 00  ss
# si l'expression est invalide, on lui renvoie le format
#################################################################
proc conv_tps { var } {
   set val 0
   if {[regexp {^[ ]*([0-9]+)[ ]*:[ ]*([0-9]+)[ ]*:[ ]*([0-9]+)[ ]*} $var phr hh mm ss] == 1} {
      set val [ expr $ss+($mm*60)+($hh*3600) ]
   } elseif {[regexp {^[ ]*([0-9]+)[ ]*:[ ]*([0-9]+)[ ]*} $var phr mm ss] == 1} {
      set val [ expr $ss+($mm*60) ]
   } elseif {[regexp {^[ ]*([0-9]+)[ ]*} $var phr ss] == 1} {
      set val [ expr $ss ]
   } else {
      if { $ashare::dbg } {
         ashare::log "<DEBUG> (conv_tps) Format incorrect pour $var"
      }
      return 0
   }
   return $val
}

# traitement des arguments
#################################################################
proc traite_argv { } {
	global argc argv
	for {set i 0} {$i < $argc} {incr i} {
		switch -exact -- [lindex $argv $i] {
			-h -
         --help {
				set ashare::fen_log 0
            ashare::mess "info" 10
            ashare::my_exit 1
			}
			-g -
         --debug {
				set ashare::dbg 1
				if { $i < [ expr $argc - 1 ] } {
					incr i
					if { [string equal [string index [lindex $argv $i] 0] - ] } {
						incr i -1
					} else {
						set ashare::dbg [lindex $argv $i]
                  set astk::config(-1,dbglevel) $ashare::dbg
					}

				}
			}
			--serv {
				if { $i < [ expr $argc - 1 ] } {
					incr i
               set astk::ihm(serv_ini) [lindex $argv $i]
				}
			}
			--profil {
				if { $i < [ expr $argc - 1 ] } {
					incr i
               set astk::ihm(profil_ini) [lindex $argv $i]
               if { [string index $astk::ihm(profil_ini) 0] != "/" } {
                  set astk::ihm(profil_ini) [file join [pwd] $astk::ihm(profil_ini)]
                  regsub -all {/\./} $astk::ihm(profil_ini) "" astk::ihm(profil_ini)
               }
				}
			}
         --agla {
				if { $i < [ expr $argc - 1 ] } {
					incr i
					if { [file exists [lindex $argv $i]] } {
						set astk::agla(fic_conf) [lindex $argv $i]
					} else {
						ashare::mess "erreur" 8 [lindex $argv $i]
					}
				}
         }
			-nb {
            set astk::ihm(style,couleur) "nb"
			}
			--check {
				set ashare::fen_log 0
				set astk::check_quit 1
			}
			--norecup {
				set astk::recup 0
			}
			--stdout {
				set ashare::fen_log 0
            destroy $astk::ihm(log)
			}
			--version {
				ashare::my_exit 0
			}
         default {
            ashare::mess "erreur" 11 [lindex $argv $i]
         }
		}
	}
}

#################################################################
proc copie_tabl { conf_src conf_dest } {
   upvar $conf_src src
   upvar $conf_dest dest
   if { $ashare::dbg >= 3 } {
      ashare::log "<DEBUG> (copie_tabl) Copie les valeurs de $conf_src dans $conf_dest" }
   set lmv [array get src]
   set nbl [expr [llength $lmv] / 2]
   for {set i 0} {$i < $nbl} {incr i} {
      set k  [expr $i * 2]
      set k1 [expr $k + 1]
      # ne pas prendre la config "locale"
      if { [regexp {^\-1} [lindex $lmv $k] mat1] != 1 } {
         set dest([lindex $lmv $k]) [lindex $lmv $k1]
      }
   }
   return 0
}   

