#########################################################################
# COPYRIGHT (C) 2003         EDF R&D              WWW.CODE-ASTER.ORG    #
#                                                                       #
# THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR         #
# MODIFY IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS        #
# PUBLISHED BY THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE    #
# LICENSE, OR (AT YOUR OPTION) ANY LATER VERSION.                       #
# THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,       #
# BUT WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF        #
# MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU      #
# GENERAL PUBLIC LICENSE FOR MORE DETAILS.                              #
#                                                                       #
# YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE     #
# ALONG WITH THIS PROGRAM; IF NOT, WRITE TO : EDF R&D CODE_ASTER,       #
#    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.        #
#########################################################################

# $Id: preferences.tcl,v 1.17 2003/05/05 15:28:51 mcourtoi Exp $

#################################################################
proc ashare::init_prefs {} {
# lecture des prfrences utilisateur
   set iret [ ashare::lire_prefs ]
   if { $ashare::dbg } {
      ashare::log "lire_prefs : Code retour = $iret" }
   if { $iret != 0 } {
      if { $iret == "FICH_NON_TROUVE" } {
         ashare::mess "info" 4 $astk::fic_prefs
         ashare::lire_prefs CREE_ASTKRC
      } elseif { $iret == "PB_astkrc_version" } {
         ashare::lire_prefs PB_astkrc_version
      } else {
         ashare::mess "erreur" 1 "lire_prefs"
         ashare::my_exit 4
      }
      ashare::save_prefs
   }

# vrifie que la langue choisie est connue
   for {set i 0} {$i < $ashare::llang(nb_lang)} {incr i} {
      if { $astk::config(-1,langue) == $ashare::llang(lang,$i) } {
         set ashare::lang $astk::config(-1,langue)
         break
      }
   }

# si on veut faire un peu de mnage, on pourrait virer les autres langues
}

# Lecture des prfrences utilisateur
#################################################################
proc ashare::lire_prefs { { ini NON } } {
# ini=CREE_ASTKRC, on lit dans [..]/astkrc/ : initialisation de l'environnement
# ini=NON, on lit dans ~/.astk/
   if { $ini == "PB_astkrc_version" } {
      ashare::mess "erreur" 4 $astk::fic_prefs.old
      file rename -force $astk::fic_prefs $astk::fic_prefs.old
      set ini CREE_ASTKRC
   }
   if { $ini == "CREE_ASTKRC" } {
      set fich [file join $astk::pathref astkrc prefs]
   } else {
		set ini NON
      set fich $astk::fic_prefs
   }
   if { $ashare::dbg } {
      ashare::mess "info" 9 "Lecture" $fich }

   set iret [ashare::lire_mc_val $fich mots vale nlu ]
   if { $iret != 0 } {
      return $iret }

   # phase de vrif
   # mots-cls reconnus
   set mots_cles $ashare::mots(MCS_pref)
   set pmcs [llength $mots_cles]

   # les mots-cls sont facultatifs, on initialise  _VIDE
   for {set j 0} {$j < $pmcs} {incr j} {
      set mm [lindex $ashare::mots(MCS_pref) $j]
      set astk::config(-1,$mm) ""
   }
   
   set nbmcs 0
   set nberr 0
   for {set j 1} {$j <= $nlu} {incr j} {
      #astkrc_version
      if { $mots($j) == "astkrc_version" } {
         if { $vale($j) != "$ashare::astkrc_version" } {
            ashare::mess "erreur" 2 $fich
            return PB_astkrc_version
         }
      } else {
         set iv [lsearch -exact $mots_cles $mots($j)]
         if { $iv < 0 } {
            # mot cl inconnu : ignor
            #ashare::mess "erreur" 5 $mots($j)
            #incr nberr
         } else {
            incr nbmcs
            set astk::config(-1,$mots($j)) $vale($j)
         }
      }
   }
   if { $nberr > 0 } {
      ashare::mess "erreur" 8 $fich
      ashare::my_exit 4
   }
   return 0
}

# Sauvegarde des prfrences utilisateur
#################################################################
proc ashare::save_prefs { } {
   set id [open $astk::fic_prefs w]
   puts $id "# AUTOMATICALLY GENERATED - DO NOT EDIT !"
   puts $id "astkrc_version : $ashare::astkrc_version"
   puts $id "#"
   set mots_cles $ashare::mots(MCS_pref)
   for { set k 0 } { $k < [ llength $mots_cles ] } { incr k } {
      set mcs [lindex $mots_cles $k]
      set val $astk::config(-1,$mcs)
      if { $val == "" } {
         set val "_VIDE"
      }
      puts $id "$mcs : $val"
   }
   close $id
   return 0
}

